// This file is part of KeY - Integrated Deductive Software Design 
// Copyright (C) 2001-2003 Universitaet Karlsruhe, Germany
//                         and Chalmers University of Technology, Sweden          
//
// The KeY system is protected by the GNU General Public License. 
// See LICENSE.TXT for details.
//

package de.uka.ilkd.key.gui;

import javax.swing.*;
import javax.swing.text.*;
import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.awt.event.KeyEvent;


public class ViewSelector extends JDialog {

    private JTextField maxTooltipLinesInputField;

    /** 
     * creates a new ViewSelector
     * @param parent The parent widget of this ViewSelector 
     */
    public ViewSelector(JFrame parent) {  
	super(parent, "Maximum line number for tooltips", true);
		layoutViewSelector(); 
		pack();
		setLocation(70,70);
		setVisible(true);
    }


    private void updateButtons() {

    }


    /** lays out the selector */
    protected void layoutViewSelector() {
    	int maxLinesInt = ProofSettings.THE_SETTINGS.getViewSettings().maxTooltipLines();
    	
    	getContentPane().setLayout(new BorderLayout());
	
	 	JPanel maxLines=new JPanel();
	 	maxLines.setLayout(new BoxLayout(maxLines, BoxLayout.X_AXIS));
	 	maxTooltipLinesInputField = new NumberInputField(maxLinesInt, 4);
		maxTooltipLinesInputField.setMaximumSize(new Dimension(40,30));
	 	maxLines.add(new JLabel("<html><font color=\"#000000\">"
	 			+ "Maximum size (line count) of the tooltips of applicable rules<br>"
				+ "with schema variable instantiations displayed. "
				+ "In case of longer <br>tooltips the instantiation will be suppressed."
				+ "</font></html>"));
		maxLines.add(maxTooltipLinesInputField);
	
	 	JButton okButton = new JButton("OK");
		okButton.setMnemonic(KeyEvent.VK_ENTER);
	
	 	okButton.addActionListener(new ActionListener() {
	 		public void actionPerformed(ActionEvent e) {
			    int maxSteps = Integer.parseInt(maxTooltipLinesInputField.getText());
			    ProofSettings.THE_SETTINGS.getViewSettings().setMaxTooltipLines(maxSteps);
			    
			    setVisible(false);
	 		}
		    });
	 	JButton saveButton = new JButton("Save as Default");
	
	 	saveButton.addActionListener(new ActionListener() {
	 		public void actionPerformed(ActionEvent e) {
			    int maxSteps = 
	                        Integer.parseInt(maxTooltipLinesInputField.getText());
			    ProofSettings dflt=ProofSettings.THE_SETTINGS;		    
			    dflt.getViewSettings().setMaxTooltipLines(maxSteps);
			    //temporary solution, stores more than wanted %%%% 
			    dflt.saveSettings();
			    setVisible(false);
	 		}
		    });
	 	JButton cancelButton = new JButton("Cancel");
		cancelButton.setMnemonic(KeyEvent.VK_ESCAPE);
	 	cancelButton.addActionListener(new ActionListener() {
	 		public void actionPerformed(ActionEvent e) {
			    setVisible(false);
			    dispose();
	 		}
	 	    });
	
	 	JPanel buttonPanel = new JPanel();
	 	buttonPanel.add(okButton);
	 	buttonPanel.add(saveButton);
	 	buttonPanel.add(cancelButton);
		
		getContentPane().setLayout(new BoxLayout(getContentPane(), 
							 BoxLayout.Y_AXIS));
		getContentPane().add(maxLines);
		getContentPane().add(buttonPanel);
	
		updateButtons();
	
    }
    
    

    // INNER CLASS TO READ ONLY NUMBERS FOR MAX APPs
    static class NumberDocument extends PlainDocument {

	public void insertString(int offs, String str, AttributeSet a) 
	    throws BadLocationException {
	    if (str == null) {
		return;
	    }
	    char[] upper = str.toCharArray();
	    for (int i = 0; i < upper.length; i++) {
		if (upper[i] < '0' || upper[i] > '9') {
		    return;
		} 
	    }
	    super.insertString(offs, new String(upper), a);
	}
	
    }

    static class NumberInputField extends JTextField {
	public NumberInputField(int number, int cols) {
	    super(cols);
	    setText(""+number);
	}

	protected Document createDefaultModel() {
	    return new NumberDocument();
	}
    }



}    
